﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.InvoiceApply.Invoice.Accept.Form', {
    extend: 'System.Control.EntityForm',
    xtype: 'App.Control.Kivii.Finances.InvoiceApply.Invoice.Accept.Form',
    entityType: 'Kivii.Finances.Entities.InvoiceApply',
    autoScroll: true,
    layout: {
        type: 'responsivecolumn',
        states: {
            small: 800,
            large: 0
        }
    },
    viewModel: {
        data: {
            ButtonText: '录入',
            ButtonIcon: 'x-fa fa-sign-in',
            entity: {
                Amount: 0,
                OperateTime: new Date()
            }
        },
        formulas: {
            amountChineseBig: function(get) {
                let _amount = get('entity.Amount');
                return KiviiContext.Const.FinancesFunctionsMoneyToChineseBig(_amount);
            }
        }
    },
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        items: ['->', {
            iconAlign: 'left',
            bind: {
                text: '{ButtonText}',
                iconCls: '{ButtonIcon}',
                disabled: '{!formIsValid}'
            },
            handler: '_onSaveEntity'
        }]
    }],
    defaults: {
        // margin: '0 10 10 10',
        labelWidth: 60,
        labelAlign: 'right',
        xtype: 'textfield'
    },
    items: [{
        xtype: 'textfield',
        responsiveCls: 'big-60 small-100',
        fieldLabel: '发票号码',
        allowBlank: false,
        bind: {
            value: '{entity.SerialNumber}'
        }
    }, {
        xtype: 'textfield',
        responsiveCls: 'big-40 small-100',
        fieldLabel: '发票代码',
        allowBlank: false,
        bind: {
            value: '{entity.SerialCode}'
        }
    }, {
        xtype: 'datefield',
        responsiveCls: 'big-60 small-100',
        fieldLabel: '开票日期',
        allowBlank: false,
        bind: {
            value: '{entity.OperateTime}'
        }
    }, {
        xtype: 'categoryField',
        fieldLabel: '发票类型',
        allowBlank: false,
        responsiveCls: 'big-40 small-100',
        internalCode: 'Type',
        readOnly: false,
        bind: {
            value: '{entity.Type}'
        }
    }, {
        fieldLabel: '开票金额',
        xtype: 'numberfield',
        readOnly: true,
        hideTrigger: true,
        minValue: 0,
        responsiveCls: 'big-100 small-100',
        step: 0.1,
        bind: '{entity.Amount}'
    }, {
        xtype: 'textfield',
        responsiveCls: 'big-100 small-100',
        readOnly: true,
        fieldLabel: '金额大写',
        bind: {
            value: '{amountChineseBig}'
        }
    }],
    setOwner: function(_item, _ctrl) {
        let _view = this;
        let _vm = _view.getViewModel();
        _vm.set('ButtonText', _ctrl.text);
        _vm.set('ButtonIcon', _ctrl.iconCls);
        _vm.set("owner", _item);

        let _entity = {};
        _entity.Amount = _item.Amount;
        _entity.OperateTime = new Date();
        _entity.Details=[];
        _entity.Details=_item.Details;
        _vm.set('entity', _entity);
    },
    controller: {
        _onSaveEntity: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _jsonData = {};
            _jsonData.ApplyKvid = _vm.data.owner.Kvid;
            _jsonData.Items = [];
            _jsonData.Items.push(_vm.data.entity);
            _view.mask('数据通讯中...');
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.InvoiceApply/Result.json',
                jsonData: _jsonData,
                success: function(_response) {
                    var _rtns = Ext.decode(_response.responseText);
                    _view.unmask();
                    Ext.toast('录入成功!');
                    _view.fireEvent('entityCreated', _controller, _rtns);
                },
                failure: function(_response) {
                    Ext.toast('录入失败!');
                    _view.unmask();
                }
            });
        }
    }
});