﻿Ext.define('App.Control.Kivii.Finances.Settlement.Chart.Month', {
    extend: 'Ext.Panel',
    xtype: 'App.Control.Kivii.Finances.Settlement.Chart.Month',
    requires: ['Package.charts', 'System.Control.ComboSelector'],
    autoScroll: true,
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
                // margin: '0 3 -10 3',
                xtype: 'displayfield',
                value: '结算月度统计',
                fieldStyle: {
                    fontSize: '22px'
                }
            }, '->', {
                xtype: 'comboSelector',
                dataSource: '/Restful/Kivii.Organizations.Entities.Organization/Query.json',
                reference: 'depart',
                responsiveCls: 'big-100 small-100',
                keyField: 'Kvid',
                valueField: 'FullName',
                key: KiviiContext.CurrentMember.DepartmentKvid,
                value: KiviiContext.CurrentMember.DepartmentName,
                editable: false,
                width: 300,
                allowBlank: true,
                listeners: {
                    select: '_onSelect'
                }
            }, {
                xtype: 'datefield',
                fieldLabel: '时间范围',
                format: 'Y-m-d',
                labelWidth: 60,
                width: 220,
                reference: 'beginTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                // fieldLabel: '结束',
                format: 'Y-m-d',
                // labelWidth: 30,
                width: 130,
                reference: 'endTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            }
        ]
    }],
    items: [{
        xtype: 'cartesian',
        reference: 'chart',
        height: 400,
        // theme: 'Muted',//'Muted',
        interactions: ['itemhighlight'],
        animation: {
            duration: 200
        },
        store: {
            fields: ['Summary', 'Amount', 'AmountPayment'],
            data: []
            // data: [{
            //         Summary: '2022-01',
            //         Amount: 420000,
            //         AmountUsed: 380000,
            //         AmountInvoice:420000
            //     },
            //     {
            //         Summary: '2022-02',
            //         Amount: 500000,
            //         AmountUsed: 400000,
            //         AmountInvoice:0
            //     },
            //     {
            //         Summary: '2022-03',
            //         Amount: 530000,
            //         AmountUsed: 300000,
            //         AmountInvoice:300000
            //     },
            //     {
            //         Summary: '2022-04',
            //         Amount: 630000,
            //         AmountUsed: 290000,
            //         AmountInvoice:0
            //     },{
            //         Summary: '2022-05',
            //         Amount: 500000,
            //         AmountUsed: 400000,
            //         AmountInvoice:500000
            //     },{
            //         Summary: '2022-06',
            //         Amount: 500000,
            //         AmountUsed: 400000,
            //         AmountInvoice:400000
            //     },{
            //         Summary: '2022-07',
            //         Amount: 500000,
            //         AmountUsed: 400000,
            //         AmountInvoice:500000
            //     }
            // ]
        },
        legend: {
            // type: 'dom',
            docked: 'bottom'
        },
        axes: [{
            type: 'numeric3d',
            position: 'left',
            fields: ['Amount', 'AmountPayment'],
            grid: true,
            renderer: 'onAxisLabelRender'
        }, {
            type: 'category3d',
            position: 'bottom',
            fields: 'Summary',
            label: {
                // font:'15px sans-serif',
                rotate: {
                    degrees: -10
                }
            }
        }],
        series: {
            type: 'bar3d',
            stacked: false,
            title: ['已结算金额', '已收费金额'],
            xField: 'Summary',
            yField: ['Amount', 'AmountPayment'],
            label: {
                field: ['Amount', 'AmountPayment'],
                display: 'insideEnd',
                renderer: 'onSeriesLabelRender'
            },
            highlight: true,
            style: {
                inGroupGapWidth: -7
            },
            tooltip: {
                trackMouse: true,
                renderer: 'onTooltipRender'
            }
        }
    }, {
        xtype: 'fieldset',
        title: '数据详情',
        collapsible: true,
        collapsed: false,
        scrollable: true,
        responsiveCls: 'big-100 small-100',
        items: [{
            xtype: 'grid',
            reference: 'detail',
            store: {
                pageSize: 10000,
                data: [],
                autoLoad: false
            },
            columns: {
                defaults: {
                    menuDisabled: true,
                    sortable: false
                },
                items: [{
                        xtype: 'rownumberer'
                    }, {
                        dataIndex: 'Summary',
                        text: '月份',
                        minWidth: 230,
                        flex: 1
                    },
                    {
                        dataIndex: 'Amount',
                        text: '已结算金额',
                        width: 120,
                        align: 'right'
                    }, {
                        dataIndex: 'AmountPayment',
                        text: '已收费金额',
                        width: 120,
                        align: 'right'
                    }
                ]
            }
        }]
    }],
    listeners: {
        boxready: '_viewReady'
    },
    controller: {
        _requestData: function(_beginTime, _endTime, _ownerKvid) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _view.getReferences();
            let _store = _refs.chart.getStore();
            _store.removeAll();
            let _detailStore = _refs.detail.getStore();
            _detailStore.removeAll();
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Settlement/PayedAnalysis.json',
                params: {
                    OwnerKvid: _ownerKvid,
                    Type: 'Month',
                    BeginTime: _beginTime, //new Date(new Date().getFullYear(), new Date().getMonth(), 1),
                    EndTime: _endTime //new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1)
                },
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    if (!_rtns || !_rtns.Results) return;
                    for (let _i = 0; _i < _rtns.Results.length; _i++) {
                        _store.add(_rtns.Results[_i]);
                        _detailStore.add(_rtns.Results[_i]);
                    }
                },
                failure: function(_response) {}
            });
        },
        _viewReady: function(_this, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _view.getReferences();
            let _store = _refs.chart.getStore();
            let _beginTime = _refs.beginTime.value;
            let _endTime = _refs.endTime.value || new Date();
            _controller._requestData(_beginTime, _endTime, KiviiContext.CurrentMember.DepartmentKvid);
        },
        _onSelect: function(_ctrl, _record, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _beginTime = _refs.beginTime.value;
            let _endTime = _refs.endTime.value || new Date();
            let _ownerKvid = _record.data.Kvid;
            if (_beginTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_beginTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            _controller._requestData(_beginTime, _endTime, _ownerKvid);
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _beginTime = _refs.beginTime.value;
            let _endTime = _refs.endTime.value || new Date();
            let _ownerKvid = _refs.depart.key;
            if (_beginTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_beginTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            _controller._requestData(_beginTime, _endTime, _ownerKvid);
        },
        onAxisLabelRender: function(axis, label, layoutContext) {
            // Custom renderer overrides the native axis label renderer.
            // Since we don't want to do anything fancy with the value
            // ourselves except adding a thousands separator, but at the same time
            // don't want to loose the formatting done by the native renderer,
            // we let the native renderer process the value first.
            var value = layoutContext.renderer(label);
            if (value >= 10000) {
                return Ext.util.Format.number(value / 10000, '0万');
            }
            return Ext.util.Format.number(value, '0元');
        },
        onTooltipRender: function(tooltip, record, item) {
            tooltip.setHtml(record.get('Summary') + ': ' +
                Ext.util.Format.number(record.get(item.field), '0,000 (元)'));
        },
        onSeriesLabelRender: function(value) {
            if (value >= 10000) {
                return Ext.util.Format.number(value / 10000, '0万');
            }
            return Ext.util.Format.number(value, '0元');
        },

        onGridColumnRender: function(v) {
            return Ext.util.Format.number(v, '0,000');
        }
    }

});