﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    public abstract class SettlementDetailBase : EntityWithMetadata
    {
        #region 关联信息,禁止更新
        [ApiMember(Description = "结算Kvid")]
        [IgnoreUpdate,NonClusteredIndex]//NonClusteredIndex标签会在创建数据表时建立索引
        [Required]
        public Guid SettlementKvid { get; set; }

        /// <summary>
        /// 冲帐关联
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OffsetKvid { get; set; }


        [ApiMember(Description = "业务Kvid,根据结算精细度,Settlement中的BizKvid可能会和SettlementDetail中的BizKvid一致,查询时需要同时查询两表数据")]
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [InternalSetter]
        [StringLength(200)]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型")]
        [InternalSetter]
        [StringLength(100)]
        public string BizType { get; set; }
        #endregion

        #region 结算明细
        [ApiMember(Description = "货品编号")]
        [StringLength(100), Default("")]
        public string GoodsId { get; set; }

        [ApiMember(Description = "条目名称，货物、服务等名称")]
        [StringLength(200), Default("")]
        public string GoodsFullName { get; set; }

        [ApiMember(Description = "条目Kvid")]
        [DefaultEmptyGuid]
        public Guid GoodsKvid { get; set; }

        [ApiMember(Description = "规格")]
        [StringLength(200), Default("")]
        public string GoodsSpecifications { get; set; }

        [ApiMember(Description = "型号")]
        [StringLength(200), Default("")]
        public string GoodsModel { get; set; }

        [ApiMember(Description = "货品单位")]
        [StringLength(50)]
        public string GoodsUnit { get; set; }

        [ApiMember(Description = "结算数量,即实际结算数量")]
        [DecimalLength(10, 2), Default(0)]
        public decimal Quantity { get; set; }

        [ApiMember(Description = "数量单位")]
        [StringLength(50), Default("")]
        public string QuantityUnit { get; set; }

        [ApiMember(Description = "单价")]
        [DecimalLength(10, 2), Default(0)]
        public decimal QuantityUnitPrice { get; set; }

        [ApiMember(Description = "预结算数量")]
        [DecimalLength(10, 2), Default(0)]
        public decimal QuantityPlan { get; set; }

        //[ApiMember(Description = "预数量单位")]
        //[StringLength(50), Default("")]
        //public string QuantityUnitPlan { get; set; }

        [ApiMember(Description = "预单价")]
        [DecimalLength(10, 2), Default(0)]
        public decimal QuantityUnitPricePlan { get; set; }

        [ApiMember(Description = "预结算金额")]
        [IgnoreUpdate]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountPlan { get; set; }

        [ApiMember(Description = "结算金额,实际结算金额")]
        [IgnoreUpdate]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "明细类型")]
        [IgnoreUpdate]
        [StringLength(50), Default("")]
        public string Type { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }
        #endregion

        #region 备注
        [ApiMember(Description = "备注")]
        [StringLength(3000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作相关  Creator Updater Operator OperateTime 

        [ApiMember(Description = "操作日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "经 办 人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string OperatorName { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }

        #endregion
    }

    [Api(Description = "结算明细,仅数据结构,不存数据库")]
    //[Alias(Configs.TableNameSettlementDetail)]
    public class SettlementDetail : SettlementDetailBase,
        IEntityHasRemark, IEntityHasBiz,
        IEntityHasCreator, IEntityHasUpdater
    {
    }

    [Alias(Configs.TableNamePrefixSettlementDetail)]
    public class EntitySettlementDetail<T> : SettlementDetailBase, IEntityInGenericTypeDb,
        IEntityHasRemark, IEntityHasBiz,
        IEntityHasCreator, IEntityHasUpdater
    {
    }
}
