﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    #region CRUDQ
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyCreate : RestfulCreate<Policy>
    {
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyUpdate : RestfulUpdate<Policy>
    {
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyDelete : RestfulDelete<Policy>
    {
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyRead : RestfulRead<Policy>
    {
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyQuery : RestfulQuery<Policy>
    {
    }


    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyDetailCreate : RestfulExecution<PolicyDetail>
    {
        public PolicyDetail Item { get; set; }

        public List<PolicyDetail> Items { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("请传入要创建的内容!");
            if (Items.IsNullOrEmpty()) Items = new List<PolicyDetail>();
            if (Item != null) Items.Add(Item);
            var rtns = new RestfulCreateResponse<PolicyDetail>();
            rtns.Results = new List<PolicyDetail>();
            var conn = KiviiContext.GetOpenedDbConnection<PolicyDetail>();
            foreach (var item in Items)
            {
                if (conn.Exists<PolicyDetail>(o => o.TargetKvid == item.TargetKvid || o.TargetName == item.TargetName)) continue;
                conn.Insert(item);
                rtns.Results.Add(item);
                var log = new EntityLog<Policy>();
                log.OwnerKvid = item.PolicyKvid;
                log.BizId = item.TargetName;
                log.BizKvid = item.Kvid;
                log.BizType = typeof(PolicyDetail).FullName;
                log.Type = "新增";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Remark = $"新增客户[{item.TargetName}],操作人:{KiviiContext.CurrentMember.FullName}";
                conn.Insert(log);
            }

            return rtns;
        }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyDetailDelete : RestfulDelete<PolicyDetail>
    {
        private List<PolicyDetail> _policyDetails = null;
        private IDbTransaction _trans = null;
        public override bool OnPreRestfulDelete(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulDeleteResponse<PolicyDetail> rtns)
        {
            _policyDetails = dbConnection.SelectByIds<PolicyDetail>(Kvids);
            if (!_policyDetails.IsNullOrEmpty()) _trans = dbConnection.OpenTransaction();
            return base.OnPreRestfulDelete(req, res, dbConnection, rtns);
        }
        public override bool OnPostRestfulDelete(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulDeleteResponse<PolicyDetail> rtns)
        {
            if (_policyDetails.IsNullOrEmpty()) return base.OnPostRestfulDelete(req, res, dbConnection, rtns);
            try
            {
                foreach (var kvid in Kvids)
                {
                    var policyDetail = _policyDetails.FirstOrDefault(p => p.Kvid == kvid);
                    if (policyDetail == null) continue;
                    var log = new EntityLog<Policy>();
                    log.OwnerKvid = policyDetail.PolicyKvid;
                    log.BizId = policyDetail.TargetName;
                    log.BizKvid = policyDetail.Kvid;
                    log.BizType = typeof(PolicyDetail).FullName;
                    log.Type = "删除";
                    log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    log.Remark = $"删除客户[{policyDetail.TargetName}],操作人:{KiviiContext.CurrentMember.FullName}";
                    dbConnection.Insert(log);
                }
                _trans?.Commit();
            }
            catch (Exception ex)
            {
                _trans?.Rollback();
                throw ex;
            }
            return base.OnPostRestfulDelete(req, res, dbConnection, rtns);
        }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyDetailRead : RestfulRead<PolicyDetail>
    {
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyDetailQuery : RestfulQuery<PolicyDetail>
    {
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyRecordQuery : RestfulQuery<PolicyRecord>
    {

    }
    #endregion

    [Api(Description = "政策匹配")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyMatching : RestfulExecution<Policy>
    {
        public string TargetName { get; set; }
        public Guid TargetKvid { get; set; }

        public DateTime? BizTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (TargetName.IsNullOrEmpty() && TargetKvid == Guid.Empty) throw new Exception("请传入付款单位信息!");
            if (BizTime == null) BizTime = new DateTime();
            var conn = KiviiContext.GetOpenedDbConnection<PolicyDetail>();
            var query = conn.From<PolicyDetail>();
            if (TargetKvid != Guid.Empty) query.Where(o => o.TargetKvid == TargetKvid);
            else query.Where(o => o.TargetName == TargetName);
            query.Select(o => o.PolicyKvid);
            conn.InitEntityType<Policy>();
            var policys = conn.Select<Policy>(o => Sql.In(o.Kvid, query) && o.EffectiveTime <= BizTime && o.ExpiredTime >= BizTime);
            var rtns = new RestfulQueryResponse<Policy>();
            rtns.Results = new List<Policy>();
            if (!policys.IsNullOrEmpty()) rtns.Results.AddRange(policys);
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    [Api(Description = "政策应用")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PolicyUsing : RestfulExecution<Policy>
    {

        public Guid PolicyKvid { get; set; }
        public Settlement Item { get; set; }
        public List<Settlement> Items { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("请传入要创建的内容!");
            if (Items.IsNullOrEmpty()) Items = new List<Settlement>();
            if (Item != null) Items.Add(Item);
            var conn = KiviiContext.GetOpenedDbConnection<Policy>();
            var policy = conn.SingleById<Policy>(PolicyKvid);
            var rtns = new RestfulQueryResponse<Settlement>();
            rtns.Results = new List<Settlement>();
            foreach (var item in Items)//先清除所有已存在的优惠政策
            {
                item.Category = string.Empty;
                if (item.Details.IsNullOrEmpty()) continue;
                item.Details.RemoveAll(o => o.BizType == typeof(Policy).FullName);
                item.Amount = item.Details.Sum(o => o.Amount);
                item.AmountPlan = item.Details.Sum(o => o.AmountPlan);
                item.RemoveAllOnlyProperties();
            }
            if (policy == null)
            {
                rtns.Results.AddRange(Items);
                rtns.Total = rtns.Results.Count;
                return rtns;
            }

            if (policy.Type == PolicyType.Original)
            {
                foreach (var item in Items)
                {
                    if (item.Details.IsNullOrEmpty()) item.Details = new List<SettlementDetail>();
                    decimal amount = 0;
                    decimal amountPlan = 0;
                    if (policy.Amount > 0)
                    {
                        if (item.AmountPlan - policy.Amount >= item.Amount)
                        {
                            item.RemoveAllOnlyProperties();
                            rtns.Results.Add(item);
                            continue;//由此可证明此优惠政策后不如原优惠力度所以跳过
                        }
                        var amountPolicy = item.AmountPlan - policy.Amount;
                        if (amountPolicy < 0) amountPolicy = 0;
                        amount = amountPolicy - item.Amount;
                        amountPlan = 0;
                    }
                    else
                    {
                        if (item.AmountPlan - item.AmountPlan * (1 - (policy.DiscountRate == null ? 1 : policy.DiscountRate.Value)) >= item.Amount)
                        {
                            item.RemoveAllOnlyProperties();
                            rtns.Results.Add(item);
                            continue;//由此可证明此优惠政策后不如原优惠力度所以跳过
                        }
                        amount = item.AmountPlan - item.AmountPlan * (1 - (policy.DiscountRate == null ? 1 : policy.DiscountRate.Value)) - item.Amount;
                        amountPlan = 0;
                    }
                    item.Category = policy.Type.ToString();
                    var detail = new SettlementDetail();
                    detail.OperateTime = DateTime.Now;
                    detail.OperatorName = KiviiContext.CurrentMember.FullName;
                    detail.OperatorKvid = KiviiContext.CurrentMember.Kvid;

                    detail.BizId = policy.Name;
                    detail.BizKvid = policy.Kvid;
                    detail.BizType = typeof(Policy).FullName;
                    detail.Amount = amount;
                    detail.AmountPlan = amountPlan;
                    detail.GoodsFullName = policy.Name;
                    detail.GoodsUnit = "次";
                    detail.Quantity = 1;
                    detail.QuantityUnitPrice = detail.Amount / detail.Quantity;
                    detail.QuantityPlan = detail.Quantity;
                    detail.QuantityUnitPricePlan = detail.AmountPlan / detail.QuantityPlan;
                    detail.QuantityUnit = "次";
                    detail.Type = "Policy";
                    detail.Currency = CurrencyUnit.CNY;
                    detail.Remark = $"执行优惠政策[{policy.Name}],减免{Math.Abs(detail.Amount)}元";
                    item.Details.Add(detail);
                    item.Amount = item.Details.Sum(o => o.Amount);
                    item.AmountPlan = item.Details.Sum(o => o.AmountPlan);
                    item.RemoveAllOnlyProperties();
                    rtns.Results.Add(item);
                }
            }
            if (policy.Type == PolicyType.Additional)
            {
                foreach (var item in Items)
                {
                    item.Category = policy.Type.ToString();
                    if (item.Details.IsNullOrEmpty()) item.Details = new List<SettlementDetail>();
                    decimal amount = 0;
                    decimal amountPlan = 0;
                    if (policy.Amount > 0)
                    {
                        amount = 0 - policy.Amount;
                        if (item.Amount - policy.Amount < 0) amount = 0 - item.Amount;
                        amountPlan = 0;
                    }
                    else
                    {
                        amount = 0 - item.Amount * (1 - (policy.DiscountRate == null ? 1 : policy.DiscountRate.Value));
                        amountPlan = 0;
                    }
                    var detail = new SettlementDetail();
                    detail.OperateTime = DateTime.Now;
                    detail.OperatorName = KiviiContext.CurrentMember.FullName;
                    detail.OperatorKvid = KiviiContext.CurrentMember.Kvid;

                    detail.BizId = policy.Name;
                    detail.BizKvid = policy.Kvid;
                    detail.BizType = typeof(Policy).FullName;
                    detail.Amount = amount;
                    detail.AmountPlan = amountPlan;
                    detail.GoodsFullName = policy.Name;
                    detail.GoodsUnit = "次";
                    detail.Quantity = 1;
                    detail.QuantityUnitPrice = detail.Amount / detail.Quantity;
                    detail.QuantityPlan = detail.Quantity;
                    detail.QuantityUnitPricePlan = detail.AmountPlan / detail.QuantityPlan;
                    detail.QuantityUnit = "次";
                    detail.Type = "Policy";
                    detail.Currency = CurrencyUnit.CNY;
                    item.Details.Add(detail);
                    item.Amount = item.Details.Sum(o => o.Amount);
                    item.AmountPlan = item.Details.Sum(o => o.AmountPlan);
                    item.RemoveAllOnlyProperties();
                    rtns.Results.Add(item);
                }
            }

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
}
